# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import RunwayMLError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import (
        tasks,
        uploads,
        organization,
        sound_effect,
        text_to_image,
        text_to_video,
        voice_dubbing,
        image_to_video,
        text_to_speech,
        video_to_video,
        voice_isolation,
        speech_to_speech,
        character_performance,
    )
    from .resources.tasks import TasksResource, AsyncTasksResource
    from .resources.uploads import UploadsResource, AsyncUploadsResource
    from .resources.organization import OrganizationResource, AsyncOrganizationResource
    from .resources.sound_effect import SoundEffectResource, AsyncSoundEffectResource
    from .resources.text_to_image import TextToImageResource, AsyncTextToImageResource
    from .resources.text_to_video import TextToVideoResource, AsyncTextToVideoResource
    from .resources.voice_dubbing import VoiceDubbingResource, AsyncVoiceDubbingResource
    from .resources.image_to_video import ImageToVideoResource, AsyncImageToVideoResource
    from .resources.text_to_speech import TextToSpeechResource, AsyncTextToSpeechResource
    from .resources.video_to_video import VideoToVideoResource, AsyncVideoToVideoResource
    from .resources.voice_isolation import VoiceIsolationResource, AsyncVoiceIsolationResource
    from .resources.speech_to_speech import SpeechToSpeechResource, AsyncSpeechToSpeechResource
    from .resources.character_performance import CharacterPerformanceResource, AsyncCharacterPerformanceResource

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "RunwayML",
    "AsyncRunwayML",
    "Client",
    "AsyncClient",
]


class RunwayML(SyncAPIClient):
    # client options
    api_key: str
    runway_version: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        runway_version: str | None = "2024-11-06",
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous RunwayML client instance.

        This automatically infers the `api_key` argument from the `RUNWAYML_API_SECRET` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("RUNWAYML_API_SECRET")
        if api_key is None:
            raise RunwayMLError(
                "The api_key client option must be set either by passing api_key to the client or by setting the RUNWAYML_API_SECRET environment variable"
            )
        self.api_key = api_key

        if runway_version is None:
            runway_version = "2024-11-06"
        self.runway_version = runway_version

        if base_url is None:
            base_url = os.environ.get("RUNWAYML_BASE_URL")
        if base_url is None:
            base_url = f"https://api.dev.runwayml.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def tasks(self) -> TasksResource:
        """Endpoints for managing tasks that have been submitted."""
        from .resources.tasks import TasksResource

        return TasksResource(self)

    @cached_property
    def uploads(self) -> UploadsResource:
        from .resources.uploads import UploadsResource

        return UploadsResource(self)

    @cached_property
    def image_to_video(self) -> ImageToVideoResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.image_to_video import ImageToVideoResource

        return ImageToVideoResource(self)

    @cached_property
    def video_to_video(self) -> VideoToVideoResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.video_to_video import VideoToVideoResource

        return VideoToVideoResource(self)

    @cached_property
    def text_to_video(self) -> TextToVideoResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.text_to_video import TextToVideoResource

        return TextToVideoResource(self)

    @cached_property
    def text_to_image(self) -> TextToImageResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.text_to_image import TextToImageResource

        return TextToImageResource(self)

    @cached_property
    def character_performance(self) -> CharacterPerformanceResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.character_performance import CharacterPerformanceResource

        return CharacterPerformanceResource(self)

    @cached_property
    def text_to_speech(self) -> TextToSpeechResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.text_to_speech import TextToSpeechResource

        return TextToSpeechResource(self)

    @cached_property
    def sound_effect(self) -> SoundEffectResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.sound_effect import SoundEffectResource

        return SoundEffectResource(self)

    @cached_property
    def voice_isolation(self) -> VoiceIsolationResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.voice_isolation import VoiceIsolationResource

        return VoiceIsolationResource(self)

    @cached_property
    def voice_dubbing(self) -> VoiceDubbingResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.voice_dubbing import VoiceDubbingResource

        return VoiceDubbingResource(self)

    @cached_property
    def speech_to_speech(self) -> SpeechToSpeechResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.speech_to_speech import SpeechToSpeechResource

        return SpeechToSpeechResource(self)

    @cached_property
    def organization(self) -> OrganizationResource:
        from .resources.organization import OrganizationResource

        return OrganizationResource(self)

    @cached_property
    def with_raw_response(self) -> RunwayMLWithRawResponse:
        return RunwayMLWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RunwayMLWithStreamedResponse:
        return RunwayMLWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            "X-Runway-Version": self.runway_version,
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        runway_version: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            runway_version=runway_version or self.runway_version,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncRunwayML(AsyncAPIClient):
    # client options
    api_key: str
    runway_version: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        runway_version: str | None = "2024-11-06",
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncRunwayML client instance.

        This automatically infers the `api_key` argument from the `RUNWAYML_API_SECRET` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("RUNWAYML_API_SECRET")
        if api_key is None:
            raise RunwayMLError(
                "The api_key client option must be set either by passing api_key to the client or by setting the RUNWAYML_API_SECRET environment variable"
            )
        self.api_key = api_key

        if runway_version is None:
            runway_version = "2024-11-06"
        self.runway_version = runway_version

        if base_url is None:
            base_url = os.environ.get("RUNWAYML_BASE_URL")
        if base_url is None:
            base_url = f"https://api.dev.runwayml.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def tasks(self) -> AsyncTasksResource:
        """Endpoints for managing tasks that have been submitted."""
        from .resources.tasks import AsyncTasksResource

        return AsyncTasksResource(self)

    @cached_property
    def uploads(self) -> AsyncUploadsResource:
        from .resources.uploads import AsyncUploadsResource

        return AsyncUploadsResource(self)

    @cached_property
    def image_to_video(self) -> AsyncImageToVideoResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.image_to_video import AsyncImageToVideoResource

        return AsyncImageToVideoResource(self)

    @cached_property
    def video_to_video(self) -> AsyncVideoToVideoResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.video_to_video import AsyncVideoToVideoResource

        return AsyncVideoToVideoResource(self)

    @cached_property
    def text_to_video(self) -> AsyncTextToVideoResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.text_to_video import AsyncTextToVideoResource

        return AsyncTextToVideoResource(self)

    @cached_property
    def text_to_image(self) -> AsyncTextToImageResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.text_to_image import AsyncTextToImageResource

        return AsyncTextToImageResource(self)

    @cached_property
    def character_performance(self) -> AsyncCharacterPerformanceResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.character_performance import AsyncCharacterPerformanceResource

        return AsyncCharacterPerformanceResource(self)

    @cached_property
    def text_to_speech(self) -> AsyncTextToSpeechResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.text_to_speech import AsyncTextToSpeechResource

        return AsyncTextToSpeechResource(self)

    @cached_property
    def sound_effect(self) -> AsyncSoundEffectResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.sound_effect import AsyncSoundEffectResource

        return AsyncSoundEffectResource(self)

    @cached_property
    def voice_isolation(self) -> AsyncVoiceIsolationResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.voice_isolation import AsyncVoiceIsolationResource

        return AsyncVoiceIsolationResource(self)

    @cached_property
    def voice_dubbing(self) -> AsyncVoiceDubbingResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.voice_dubbing import AsyncVoiceDubbingResource

        return AsyncVoiceDubbingResource(self)

    @cached_property
    def speech_to_speech(self) -> AsyncSpeechToSpeechResource:
        """These endpoints all kick off tasks to create generations."""
        from .resources.speech_to_speech import AsyncSpeechToSpeechResource

        return AsyncSpeechToSpeechResource(self)

    @cached_property
    def organization(self) -> AsyncOrganizationResource:
        from .resources.organization import AsyncOrganizationResource

        return AsyncOrganizationResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncRunwayMLWithRawResponse:
        return AsyncRunwayMLWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRunwayMLWithStreamedResponse:
        return AsyncRunwayMLWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            "X-Runway-Version": self.runway_version,
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        runway_version: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            runway_version=runway_version or self.runway_version,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class RunwayMLWithRawResponse:
    _client: RunwayML

    def __init__(self, client: RunwayML) -> None:
        self._client = client

    @cached_property
    def tasks(self) -> tasks.TasksResourceWithRawResponse:
        """Endpoints for managing tasks that have been submitted."""
        from .resources.tasks import TasksResourceWithRawResponse

        return TasksResourceWithRawResponse(self._client.tasks)

    @cached_property
    def uploads(self) -> uploads.UploadsResourceWithRawResponse:
        from .resources.uploads import UploadsResourceWithRawResponse

        return UploadsResourceWithRawResponse(self._client.uploads)

    @cached_property
    def image_to_video(self) -> image_to_video.ImageToVideoResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.image_to_video import ImageToVideoResourceWithRawResponse

        return ImageToVideoResourceWithRawResponse(self._client.image_to_video)

    @cached_property
    def video_to_video(self) -> video_to_video.VideoToVideoResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.video_to_video import VideoToVideoResourceWithRawResponse

        return VideoToVideoResourceWithRawResponse(self._client.video_to_video)

    @cached_property
    def text_to_video(self) -> text_to_video.TextToVideoResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.text_to_video import TextToVideoResourceWithRawResponse

        return TextToVideoResourceWithRawResponse(self._client.text_to_video)

    @cached_property
    def text_to_image(self) -> text_to_image.TextToImageResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.text_to_image import TextToImageResourceWithRawResponse

        return TextToImageResourceWithRawResponse(self._client.text_to_image)

    @cached_property
    def character_performance(self) -> character_performance.CharacterPerformanceResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.character_performance import CharacterPerformanceResourceWithRawResponse

        return CharacterPerformanceResourceWithRawResponse(self._client.character_performance)

    @cached_property
    def text_to_speech(self) -> text_to_speech.TextToSpeechResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.text_to_speech import TextToSpeechResourceWithRawResponse

        return TextToSpeechResourceWithRawResponse(self._client.text_to_speech)

    @cached_property
    def sound_effect(self) -> sound_effect.SoundEffectResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.sound_effect import SoundEffectResourceWithRawResponse

        return SoundEffectResourceWithRawResponse(self._client.sound_effect)

    @cached_property
    def voice_isolation(self) -> voice_isolation.VoiceIsolationResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.voice_isolation import VoiceIsolationResourceWithRawResponse

        return VoiceIsolationResourceWithRawResponse(self._client.voice_isolation)

    @cached_property
    def voice_dubbing(self) -> voice_dubbing.VoiceDubbingResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.voice_dubbing import VoiceDubbingResourceWithRawResponse

        return VoiceDubbingResourceWithRawResponse(self._client.voice_dubbing)

    @cached_property
    def speech_to_speech(self) -> speech_to_speech.SpeechToSpeechResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.speech_to_speech import SpeechToSpeechResourceWithRawResponse

        return SpeechToSpeechResourceWithRawResponse(self._client.speech_to_speech)

    @cached_property
    def organization(self) -> organization.OrganizationResourceWithRawResponse:
        from .resources.organization import OrganizationResourceWithRawResponse

        return OrganizationResourceWithRawResponse(self._client.organization)


class AsyncRunwayMLWithRawResponse:
    _client: AsyncRunwayML

    def __init__(self, client: AsyncRunwayML) -> None:
        self._client = client

    @cached_property
    def tasks(self) -> tasks.AsyncTasksResourceWithRawResponse:
        """Endpoints for managing tasks that have been submitted."""
        from .resources.tasks import AsyncTasksResourceWithRawResponse

        return AsyncTasksResourceWithRawResponse(self._client.tasks)

    @cached_property
    def uploads(self) -> uploads.AsyncUploadsResourceWithRawResponse:
        from .resources.uploads import AsyncUploadsResourceWithRawResponse

        return AsyncUploadsResourceWithRawResponse(self._client.uploads)

    @cached_property
    def image_to_video(self) -> image_to_video.AsyncImageToVideoResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.image_to_video import AsyncImageToVideoResourceWithRawResponse

        return AsyncImageToVideoResourceWithRawResponse(self._client.image_to_video)

    @cached_property
    def video_to_video(self) -> video_to_video.AsyncVideoToVideoResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.video_to_video import AsyncVideoToVideoResourceWithRawResponse

        return AsyncVideoToVideoResourceWithRawResponse(self._client.video_to_video)

    @cached_property
    def text_to_video(self) -> text_to_video.AsyncTextToVideoResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.text_to_video import AsyncTextToVideoResourceWithRawResponse

        return AsyncTextToVideoResourceWithRawResponse(self._client.text_to_video)

    @cached_property
    def text_to_image(self) -> text_to_image.AsyncTextToImageResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.text_to_image import AsyncTextToImageResourceWithRawResponse

        return AsyncTextToImageResourceWithRawResponse(self._client.text_to_image)

    @cached_property
    def character_performance(self) -> character_performance.AsyncCharacterPerformanceResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.character_performance import AsyncCharacterPerformanceResourceWithRawResponse

        return AsyncCharacterPerformanceResourceWithRawResponse(self._client.character_performance)

    @cached_property
    def text_to_speech(self) -> text_to_speech.AsyncTextToSpeechResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.text_to_speech import AsyncTextToSpeechResourceWithRawResponse

        return AsyncTextToSpeechResourceWithRawResponse(self._client.text_to_speech)

    @cached_property
    def sound_effect(self) -> sound_effect.AsyncSoundEffectResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.sound_effect import AsyncSoundEffectResourceWithRawResponse

        return AsyncSoundEffectResourceWithRawResponse(self._client.sound_effect)

    @cached_property
    def voice_isolation(self) -> voice_isolation.AsyncVoiceIsolationResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.voice_isolation import AsyncVoiceIsolationResourceWithRawResponse

        return AsyncVoiceIsolationResourceWithRawResponse(self._client.voice_isolation)

    @cached_property
    def voice_dubbing(self) -> voice_dubbing.AsyncVoiceDubbingResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.voice_dubbing import AsyncVoiceDubbingResourceWithRawResponse

        return AsyncVoiceDubbingResourceWithRawResponse(self._client.voice_dubbing)

    @cached_property
    def speech_to_speech(self) -> speech_to_speech.AsyncSpeechToSpeechResourceWithRawResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.speech_to_speech import AsyncSpeechToSpeechResourceWithRawResponse

        return AsyncSpeechToSpeechResourceWithRawResponse(self._client.speech_to_speech)

    @cached_property
    def organization(self) -> organization.AsyncOrganizationResourceWithRawResponse:
        from .resources.organization import AsyncOrganizationResourceWithRawResponse

        return AsyncOrganizationResourceWithRawResponse(self._client.organization)


class RunwayMLWithStreamedResponse:
    _client: RunwayML

    def __init__(self, client: RunwayML) -> None:
        self._client = client

    @cached_property
    def tasks(self) -> tasks.TasksResourceWithStreamingResponse:
        """Endpoints for managing tasks that have been submitted."""
        from .resources.tasks import TasksResourceWithStreamingResponse

        return TasksResourceWithStreamingResponse(self._client.tasks)

    @cached_property
    def uploads(self) -> uploads.UploadsResourceWithStreamingResponse:
        from .resources.uploads import UploadsResourceWithStreamingResponse

        return UploadsResourceWithStreamingResponse(self._client.uploads)

    @cached_property
    def image_to_video(self) -> image_to_video.ImageToVideoResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.image_to_video import ImageToVideoResourceWithStreamingResponse

        return ImageToVideoResourceWithStreamingResponse(self._client.image_to_video)

    @cached_property
    def video_to_video(self) -> video_to_video.VideoToVideoResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.video_to_video import VideoToVideoResourceWithStreamingResponse

        return VideoToVideoResourceWithStreamingResponse(self._client.video_to_video)

    @cached_property
    def text_to_video(self) -> text_to_video.TextToVideoResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.text_to_video import TextToVideoResourceWithStreamingResponse

        return TextToVideoResourceWithStreamingResponse(self._client.text_to_video)

    @cached_property
    def text_to_image(self) -> text_to_image.TextToImageResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.text_to_image import TextToImageResourceWithStreamingResponse

        return TextToImageResourceWithStreamingResponse(self._client.text_to_image)

    @cached_property
    def character_performance(self) -> character_performance.CharacterPerformanceResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.character_performance import CharacterPerformanceResourceWithStreamingResponse

        return CharacterPerformanceResourceWithStreamingResponse(self._client.character_performance)

    @cached_property
    def text_to_speech(self) -> text_to_speech.TextToSpeechResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.text_to_speech import TextToSpeechResourceWithStreamingResponse

        return TextToSpeechResourceWithStreamingResponse(self._client.text_to_speech)

    @cached_property
    def sound_effect(self) -> sound_effect.SoundEffectResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.sound_effect import SoundEffectResourceWithStreamingResponse

        return SoundEffectResourceWithStreamingResponse(self._client.sound_effect)

    @cached_property
    def voice_isolation(self) -> voice_isolation.VoiceIsolationResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.voice_isolation import VoiceIsolationResourceWithStreamingResponse

        return VoiceIsolationResourceWithStreamingResponse(self._client.voice_isolation)

    @cached_property
    def voice_dubbing(self) -> voice_dubbing.VoiceDubbingResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.voice_dubbing import VoiceDubbingResourceWithStreamingResponse

        return VoiceDubbingResourceWithStreamingResponse(self._client.voice_dubbing)

    @cached_property
    def speech_to_speech(self) -> speech_to_speech.SpeechToSpeechResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.speech_to_speech import SpeechToSpeechResourceWithStreamingResponse

        return SpeechToSpeechResourceWithStreamingResponse(self._client.speech_to_speech)

    @cached_property
    def organization(self) -> organization.OrganizationResourceWithStreamingResponse:
        from .resources.organization import OrganizationResourceWithStreamingResponse

        return OrganizationResourceWithStreamingResponse(self._client.organization)


class AsyncRunwayMLWithStreamedResponse:
    _client: AsyncRunwayML

    def __init__(self, client: AsyncRunwayML) -> None:
        self._client = client

    @cached_property
    def tasks(self) -> tasks.AsyncTasksResourceWithStreamingResponse:
        """Endpoints for managing tasks that have been submitted."""
        from .resources.tasks import AsyncTasksResourceWithStreamingResponse

        return AsyncTasksResourceWithStreamingResponse(self._client.tasks)

    @cached_property
    def uploads(self) -> uploads.AsyncUploadsResourceWithStreamingResponse:
        from .resources.uploads import AsyncUploadsResourceWithStreamingResponse

        return AsyncUploadsResourceWithStreamingResponse(self._client.uploads)

    @cached_property
    def image_to_video(self) -> image_to_video.AsyncImageToVideoResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.image_to_video import AsyncImageToVideoResourceWithStreamingResponse

        return AsyncImageToVideoResourceWithStreamingResponse(self._client.image_to_video)

    @cached_property
    def video_to_video(self) -> video_to_video.AsyncVideoToVideoResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.video_to_video import AsyncVideoToVideoResourceWithStreamingResponse

        return AsyncVideoToVideoResourceWithStreamingResponse(self._client.video_to_video)

    @cached_property
    def text_to_video(self) -> text_to_video.AsyncTextToVideoResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.text_to_video import AsyncTextToVideoResourceWithStreamingResponse

        return AsyncTextToVideoResourceWithStreamingResponse(self._client.text_to_video)

    @cached_property
    def text_to_image(self) -> text_to_image.AsyncTextToImageResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.text_to_image import AsyncTextToImageResourceWithStreamingResponse

        return AsyncTextToImageResourceWithStreamingResponse(self._client.text_to_image)

    @cached_property
    def character_performance(self) -> character_performance.AsyncCharacterPerformanceResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.character_performance import AsyncCharacterPerformanceResourceWithStreamingResponse

        return AsyncCharacterPerformanceResourceWithStreamingResponse(self._client.character_performance)

    @cached_property
    def text_to_speech(self) -> text_to_speech.AsyncTextToSpeechResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.text_to_speech import AsyncTextToSpeechResourceWithStreamingResponse

        return AsyncTextToSpeechResourceWithStreamingResponse(self._client.text_to_speech)

    @cached_property
    def sound_effect(self) -> sound_effect.AsyncSoundEffectResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.sound_effect import AsyncSoundEffectResourceWithStreamingResponse

        return AsyncSoundEffectResourceWithStreamingResponse(self._client.sound_effect)

    @cached_property
    def voice_isolation(self) -> voice_isolation.AsyncVoiceIsolationResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.voice_isolation import AsyncVoiceIsolationResourceWithStreamingResponse

        return AsyncVoiceIsolationResourceWithStreamingResponse(self._client.voice_isolation)

    @cached_property
    def voice_dubbing(self) -> voice_dubbing.AsyncVoiceDubbingResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.voice_dubbing import AsyncVoiceDubbingResourceWithStreamingResponse

        return AsyncVoiceDubbingResourceWithStreamingResponse(self._client.voice_dubbing)

    @cached_property
    def speech_to_speech(self) -> speech_to_speech.AsyncSpeechToSpeechResourceWithStreamingResponse:
        """These endpoints all kick off tasks to create generations."""
        from .resources.speech_to_speech import AsyncSpeechToSpeechResourceWithStreamingResponse

        return AsyncSpeechToSpeechResourceWithStreamingResponse(self._client.speech_to_speech)

    @cached_property
    def organization(self) -> organization.AsyncOrganizationResourceWithStreamingResponse:
        from .resources.organization import AsyncOrganizationResourceWithStreamingResponse

        return AsyncOrganizationResourceWithStreamingResponse(self._client.organization)


Client = RunwayML

AsyncClient = AsyncRunwayML



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["VoiceDubbingCreateParams"]


class VoiceDubbingCreateParams(TypedDict, total=False):
    audio_uri: Required[Annotated[str, PropertyInfo(alias="audioUri")]]
    """A HTTPS URL."""

    model: Required[Literal["eleven_voice_dubbing"]]

    target_lang: Required[
        Annotated[
            Literal[
                "en",
                "hi",
                "pt",
                "zh",
                "es",
                "fr",
                "de",
                "ja",
                "ar",
                "ru",
                "ko",
                "id",
                "it",
                "nl",
                "tr",
                "pl",
                "sv",
                "fil",
                "ms",
                "ro",
                "uk",
                "el",
                "cs",
                "da",
                "fi",
                "bg",
                "hr",
                "sk",
                "ta",
            ],
            PropertyInfo(alias="targetLang"),
        ]
    ]
    """
    The target language code to dub the audio to (e.g., "es" for Spanish, "fr" for
    French).
    """

    disable_voice_cloning: Annotated[bool, PropertyInfo(alias="disableVoiceCloning")]
    """Whether to disable voice cloning and use a generic voice instead."""

    drop_background_audio: Annotated[bool, PropertyInfo(alias="dropBackgroundAudio")]
    """Whether to remove background audio from the dubbed output."""

    num_speakers: Annotated[int, PropertyInfo(alias="numSpeakers")]
    """The number of speakers in the audio.

    If not provided, it will be detected automatically.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, overload

import httpx

from ..types import text_to_video_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import required_args, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..lib.polling import (
    NewTaskCreatedResponse,
    AsyncNewTaskCreatedResponse,
    create_waitable_resource,
    create_async_waitable_resource,
)
from .._base_client import make_request_options
from ..types.text_to_video_create_response import TextToVideoCreateResponse

__all__ = ["TextToVideoResource", "AsyncTextToVideoResource"]


class TextToVideoResource(SyncAPIResource):
    """These endpoints all kick off tasks to create generations."""

    @cached_property
    def with_raw_response(self) -> TextToVideoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return TextToVideoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TextToVideoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return TextToVideoResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        duration: int,
        model: Literal["gen4.5"],
        prompt_text: str,
        ratio: Literal["1280:720", "720:1280"],
        content_moderation: text_to_video_create_params.Gen4_5ContentModeration | Omit = omit,
        seed: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from a text prompt.

        Args:
          duration: The number of seconds of duration for the output video. Must be an integer from
              2 to 10.

          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          ratio: The resolution of the output video.

          content_moderation: Settings that affect the behavior of the content moderation system.

          seed: If unspecified, a random number is chosen. Varying the seed integer is a way to
              get different results for the same other request parameters. Using the same seed
              integer for an identical request will produce similar results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        model: Literal["veo3.1"],
        prompt_text: str,
        ratio: Literal["1280:720", "720:1280", "1080:1920", "1920:1080"],
        audio: bool | Omit = omit,
        duration: Literal[4, 6, 8] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from a text prompt.

        Args:
          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          ratio: The resolution of the output video.

          audio: Whether to generate audio for the video. Audio inclusion affects pricing.

          duration: The number of seconds of duration for the output video.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        model: Literal["veo3.1_fast"],
        prompt_text: str,
        ratio: Literal["1280:720", "720:1280", "1080:1920", "1920:1080"],
        audio: bool | Omit = omit,
        duration: Literal[4, 6, 8] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from a text prompt.

        Args:
          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          ratio: The resolution of the output video.

          audio: Whether to generate audio for the video. Audio inclusion affects pricing.

          duration: The number of seconds of duration for the output video.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        duration: Literal[8],
        model: Literal["veo3"],
        prompt_text: str,
        ratio: Literal["1280:720", "720:1280", "1080:1920", "1920:1080"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from a text prompt.

        Args:
          duration: The number of seconds of duration for the output video.

          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          ratio: The resolution of the output video.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["duration", "model", "prompt_text", "ratio"], ["model", "prompt_text", "ratio"])
    def create(
        self,
        *,
        duration: int | Literal[4, 6, 8] | Literal[8] | Omit = omit,
        model: Literal["gen4.5"] | Literal["veo3.1"] | Literal["veo3.1_fast"] | Literal["veo3"],
        prompt_text: str,
        ratio: Literal["1280:720", "720:1280"] | Literal["1280:720", "720:1280", "1080:1920", "1920:1080"],
        content_moderation: text_to_video_create_params.Gen4_5ContentModeration | Omit = omit,
        seed: int | Omit = omit,
        audio: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        return self._post(
            "/v1/text_to_video",
            body=maybe_transform(
                {
                    "duration": duration,
                    "model": model,
                    "prompt_text": prompt_text,
                    "ratio": ratio,
                    "content_moderation": content_moderation,
                    "seed": seed,
                    "audio": audio,
                },
                text_to_video_create_params.TextToVideoCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=create_waitable_resource(TextToVideoCreateResponse, self._client),
        )


class AsyncTextToVideoResource(AsyncAPIResource):
    """These endpoints all kick off tasks to create generations."""

    @cached_property
    def with_raw_response(self) -> AsyncTextToVideoResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTextToVideoResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTextToVideoResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return AsyncTextToVideoResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        duration: int,
        model: Literal["gen4.5"],
        prompt_text: str,
        ratio: Literal["1280:720", "720:1280"],
        content_moderation: text_to_video_create_params.Gen4_5ContentModeration | Omit = omit,
        seed: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from a text prompt.

        Args:
          duration: The number of seconds of duration for the output video. Must be an integer from
              2 to 10.

          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          ratio: The resolution of the output video.

          content_moderation: Settings that affect the behavior of the content moderation system.

          seed: If unspecified, a random number is chosen. Varying the seed integer is a way to
              get different results for the same other request parameters. Using the same seed
              integer for an identical request will produce similar results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        model: Literal["veo3.1"],
        prompt_text: str,
        ratio: Literal["1280:720", "720:1280", "1080:1920", "1920:1080"],
        audio: bool | Omit = omit,
        duration: Literal[4, 6, 8] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from a text prompt.

        Args:
          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          ratio: The resolution of the output video.

          audio: Whether to generate audio for the video. Audio inclusion affects pricing.

          duration: The number of seconds of duration for the output video.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        model: Literal["veo3.1_fast"],
        prompt_text: str,
        ratio: Literal["1280:720", "720:1280", "1080:1920", "1920:1080"],
        audio: bool | Omit = omit,
        duration: Literal[4, 6, 8] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from a text prompt.

        Args:
          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          ratio: The resolution of the output video.

          audio: Whether to generate audio for the video. Audio inclusion affects pricing.

          duration: The number of seconds of duration for the output video.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        duration: Literal[8],
        model: Literal["veo3"],
        prompt_text: str,
        ratio: Literal["1280:720", "720:1280", "1080:1920", "1920:1080"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        """
        This endpoint will start a new task to generate a video from a text prompt.

        Args:
          duration: The number of seconds of duration for the output video.

          prompt_text: A non-empty string up to 1000 characters (measured in UTF-16 code units). This
              should describe in detail what should appear in the output.

          ratio: The resolution of the output video.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["duration", "model", "prompt_text", "ratio"], ["model", "prompt_text", "ratio"])
    async def create(
        self,
        *,
        duration: int | Literal[4, 6, 8] | Literal[8] | Omit = omit,
        model: Literal["gen4.5"] | Literal["veo3.1"] | Literal["veo3.1_fast"] | Literal["veo3"],
        prompt_text: str,
        ratio: Literal["1280:720", "720:1280"] | Literal["1280:720", "720:1280", "1080:1920", "1920:1080"],
        content_moderation: text_to_video_create_params.Gen4_5ContentModeration | Omit = omit,
        seed: int | Omit = omit,
        audio: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        return await self._post(
            "/v1/text_to_video",
            body=await async_maybe_transform(
                {
                    "duration": duration,
                    "model": model,
                    "prompt_text": prompt_text,
                    "ratio": ratio,
                    "content_moderation": content_moderation,
                    "seed": seed,
                    "audio": audio,
                },
                text_to_video_create_params.TextToVideoCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=create_async_waitable_resource(TextToVideoCreateResponse, self._client),
        )


class TextToVideoResourceWithRawResponse:
    def __init__(self, text_to_video: TextToVideoResource) -> None:
        self._text_to_video = text_to_video

        self.create = to_raw_response_wrapper(
            text_to_video.create,
        )


class AsyncTextToVideoResourceWithRawResponse:
    def __init__(self, text_to_video: AsyncTextToVideoResource) -> None:
        self._text_to_video = text_to_video

        self.create = async_to_raw_response_wrapper(
            text_to_video.create,
        )


class TextToVideoResourceWithStreamingResponse:
    def __init__(self, text_to_video: TextToVideoResource) -> None:
        self._text_to_video = text_to_video

        self.create = to_streamed_response_wrapper(
            text_to_video.create,
        )


class AsyncTextToVideoResourceWithStreamingResponse:
    def __init__(self, text_to_video: AsyncTextToVideoResource) -> None:
        self._text_to_video = text_to_video

        self.create = async_to_streamed_response_wrapper(
            text_to_video.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["VoiceDubbingCreateResponse"]


class VoiceDubbingCreateResponse(BaseModel):
    id: str
    """The ID of the task that was created. Use this to retrieve the task later."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import voice_dubbing_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..lib.polling import (
    NewTaskCreatedResponse,
    AsyncNewTaskCreatedResponse,
    create_waitable_resource,
    create_async_waitable_resource,
)
from .._base_client import make_request_options
from ..types.voice_dubbing_create_response import VoiceDubbingCreateResponse

__all__ = ["VoiceDubbingResource", "AsyncVoiceDubbingResource"]


class VoiceDubbingResource(SyncAPIResource):
    """These endpoints all kick off tasks to create generations."""

    @cached_property
    def with_raw_response(self) -> VoiceDubbingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return VoiceDubbingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VoiceDubbingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return VoiceDubbingResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        audio_uri: str,
        model: Literal["eleven_voice_dubbing"],
        target_lang: Literal[
            "en",
            "hi",
            "pt",
            "zh",
            "es",
            "fr",
            "de",
            "ja",
            "ar",
            "ru",
            "ko",
            "id",
            "it",
            "nl",
            "tr",
            "pl",
            "sv",
            "fil",
            "ms",
            "ro",
            "uk",
            "el",
            "cs",
            "da",
            "fi",
            "bg",
            "hr",
            "sk",
            "ta",
        ],
        disable_voice_cloning: bool | Omit = omit,
        drop_background_audio: bool | Omit = omit,
        num_speakers: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        """
        This endpoint will start a new task to dub audio content to a target language.

        Args:
          audio_uri: A HTTPS URL.

          target_lang: The target language code to dub the audio to (e.g., "es" for Spanish, "fr" for
              French).

          disable_voice_cloning: Whether to disable voice cloning and use a generic voice instead.

          drop_background_audio: Whether to remove background audio from the dubbed output.

          num_speakers: The number of speakers in the audio. If not provided, it will be detected
              automatically.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/voice_dubbing",
            body=maybe_transform(
                {
                    "audio_uri": audio_uri,
                    "model": model,
                    "target_lang": target_lang,
                    "disable_voice_cloning": disable_voice_cloning,
                    "drop_background_audio": drop_background_audio,
                    "num_speakers": num_speakers,
                },
                voice_dubbing_create_params.VoiceDubbingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=create_waitable_resource(VoiceDubbingCreateResponse, self._client),
        )


class AsyncVoiceDubbingResource(AsyncAPIResource):
    """These endpoints all kick off tasks to create generations."""

    @cached_property
    def with_raw_response(self) -> AsyncVoiceDubbingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVoiceDubbingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVoiceDubbingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return AsyncVoiceDubbingResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        audio_uri: str,
        model: Literal["eleven_voice_dubbing"],
        target_lang: Literal[
            "en",
            "hi",
            "pt",
            "zh",
            "es",
            "fr",
            "de",
            "ja",
            "ar",
            "ru",
            "ko",
            "id",
            "it",
            "nl",
            "tr",
            "pl",
            "sv",
            "fil",
            "ms",
            "ro",
            "uk",
            "el",
            "cs",
            "da",
            "fi",
            "bg",
            "hr",
            "sk",
            "ta",
        ],
        disable_voice_cloning: bool | Omit = omit,
        drop_background_audio: bool | Omit = omit,
        num_speakers: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        """
        This endpoint will start a new task to dub audio content to a target language.

        Args:
          audio_uri: A HTTPS URL.

          target_lang: The target language code to dub the audio to (e.g., "es" for Spanish, "fr" for
              French).

          disable_voice_cloning: Whether to disable voice cloning and use a generic voice instead.

          drop_background_audio: Whether to remove background audio from the dubbed output.

          num_speakers: The number of speakers in the audio. If not provided, it will be detected
              automatically.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/voice_dubbing",
            body=await async_maybe_transform(
                {
                    "audio_uri": audio_uri,
                    "model": model,
                    "target_lang": target_lang,
                    "disable_voice_cloning": disable_voice_cloning,
                    "drop_background_audio": drop_background_audio,
                    "num_speakers": num_speakers,
                },
                voice_dubbing_create_params.VoiceDubbingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=create_async_waitable_resource(VoiceDubbingCreateResponse, self._client),
        )


class VoiceDubbingResourceWithRawResponse:
    def __init__(self, voice_dubbing: VoiceDubbingResource) -> None:
        self._voice_dubbing = voice_dubbing

        self.create = to_raw_response_wrapper(
            voice_dubbing.create,
        )


class AsyncVoiceDubbingResourceWithRawResponse:
    def __init__(self, voice_dubbing: AsyncVoiceDubbingResource) -> None:
        self._voice_dubbing = voice_dubbing

        self.create = async_to_raw_response_wrapper(
            voice_dubbing.create,
        )


class VoiceDubbingResourceWithStreamingResponse:
    def __init__(self, voice_dubbing: VoiceDubbingResource) -> None:
        self._voice_dubbing = voice_dubbing

        self.create = to_streamed_response_wrapper(
            voice_dubbing.create,
        )


class AsyncVoiceDubbingResourceWithStreamingResponse:
    def __init__(self, voice_dubbing: AsyncVoiceDubbingResource) -> None:
        self._voice_dubbing = voice_dubbing

        self.create = async_to_streamed_response_wrapper(
            voice_dubbing.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["OrganizationRetrieveResponse", "Tier", "TierModels", "Usage", "UsageModels"]


class TierModels(BaseModel):
    """Limits associated with the model."""

    max_concurrent_generations: int = FieldInfo(alias="maxConcurrentGenerations")
    """The maximum number of generations that can be run concurrently for this model."""

    max_daily_generations: int = FieldInfo(alias="maxDailyGenerations")
    """The maximum number of generations that can be created each day for this model."""


class Tier(BaseModel):
    """Limits associated with the organization's tier."""

    max_monthly_credit_spend: int = FieldInfo(alias="maxMonthlyCreditSpend")
    """The maximum number of credits that can be purchased in a month."""

    models: Dict[str, TierModels]
    """An object containing model-specific limits. Each key represents a model."""


class UsageModels(BaseModel):
    """Usage data for the model."""

    daily_generations: int = FieldInfo(alias="dailyGenerations")
    """The number of generations that have been run for this model in the past day."""


class Usage(BaseModel):
    """Usage data for the organization."""

    models: Dict[str, UsageModels]


class OrganizationRetrieveResponse(BaseModel):
    credit_balance: int = FieldInfo(alias="creditBalance")
    """The number of credits remaining in the organization account."""

    tier: Tier
    """Limits associated with the organization's tier."""

    usage: Usage
    """Usage data for the organization."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["OrganizationRetrieveUsageParams"]


class OrganizationRetrieveUsageParams(TypedDict, total=False):
    before_date: Annotated[Union[str, date], PropertyInfo(alias="beforeDate", format="iso8601")]
    """The end date of the usage data in ISO-8601 format (YYYY-MM-DD), not inclusive.

    If unspecified, it will default to thirty days after the start date. Must be
    less than or equal to 90 days after the start date. All dates are in UTC.
    """

    start_date: Annotated[Union[str, date], PropertyInfo(alias="startDate", format="iso8601")]
    """The start date of the usage data in ISO-8601 format (YYYY-MM-DD).

    If unspecified, it will default to 30 days before the current date. All dates
    are in UTC.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import character_performance_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..lib.polling import (
    NewTaskCreatedResponse,
    AsyncNewTaskCreatedResponse,
    create_waitable_resource,
    create_async_waitable_resource,
)
from .._base_client import make_request_options
from ..types.character_performance_create_response import CharacterPerformanceCreateResponse

__all__ = ["CharacterPerformanceResource", "AsyncCharacterPerformanceResource"]


class CharacterPerformanceResource(SyncAPIResource):
    """These endpoints all kick off tasks to create generations."""

    @cached_property
    def with_raw_response(self) -> CharacterPerformanceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return CharacterPerformanceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CharacterPerformanceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return CharacterPerformanceResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        character: character_performance_create_params.Character,
        model: Literal["act_two"],
        reference: character_performance_create_params.Reference,
        body_control: bool | Omit = omit,
        content_moderation: character_performance_create_params.ContentModeration | Omit = omit,
        expression_intensity: int | Omit = omit,
        ratio: Literal["1280:720", "720:1280", "960:960", "1104:832", "832:1104", "1584:672"] | Omit = omit,
        seed: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        """
        This endpoint will start a new task to control a character's facial expressions
        and body movements using a reference video.

        Args:
          character: The character to control. You can either provide a video or an image. A visually
              recognizable face must be visible and stay within the frame.

          reference: A video of a person performing in the manner that you would like your character
              to perform. The video must be between 3 and 30 seconds in duration.

          body_control: A boolean indicating whether to enable body control. When enabled, non-facial
              movements and gestures will be applied to the character in addition to facial
              expressions.

          content_moderation: Settings that affect the behavior of the content moderation system.

          expression_intensity: An integer between 1 and 5 (inclusive). A larger value increases the intensity
              of the character's expression.

          ratio: The resolution of the output video.

          seed: If unspecified, a random number is chosen. Varying the seed integer is a way to
              get different results for the same other request parameters. Using the same seed
              integer for an identical request will produce similar results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/character_performance",
            body=maybe_transform(
                {
                    "character": character,
                    "model": model,
                    "reference": reference,
                    "body_control": body_control,
                    "content_moderation": content_moderation,
                    "expression_intensity": expression_intensity,
                    "ratio": ratio,
                    "seed": seed,
                },
                character_performance_create_params.CharacterPerformanceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=create_waitable_resource(CharacterPerformanceCreateResponse, self._client),
        )


class AsyncCharacterPerformanceResource(AsyncAPIResource):
    """These endpoints all kick off tasks to create generations."""

    @cached_property
    def with_raw_response(self) -> AsyncCharacterPerformanceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCharacterPerformanceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCharacterPerformanceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return AsyncCharacterPerformanceResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        character: character_performance_create_params.Character,
        model: Literal["act_two"],
        reference: character_performance_create_params.Reference,
        body_control: bool | Omit = omit,
        content_moderation: character_performance_create_params.ContentModeration | Omit = omit,
        expression_intensity: int | Omit = omit,
        ratio: Literal["1280:720", "720:1280", "960:960", "1104:832", "832:1104", "1584:672"] | Omit = omit,
        seed: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        """
        This endpoint will start a new task to control a character's facial expressions
        and body movements using a reference video.

        Args:
          character: The character to control. You can either provide a video or an image. A visually
              recognizable face must be visible and stay within the frame.

          reference: A video of a person performing in the manner that you would like your character
              to perform. The video must be between 3 and 30 seconds in duration.

          body_control: A boolean indicating whether to enable body control. When enabled, non-facial
              movements and gestures will be applied to the character in addition to facial
              expressions.

          content_moderation: Settings that affect the behavior of the content moderation system.

          expression_intensity: An integer between 1 and 5 (inclusive). A larger value increases the intensity
              of the character's expression.

          ratio: The resolution of the output video.

          seed: If unspecified, a random number is chosen. Varying the seed integer is a way to
              get different results for the same other request parameters. Using the same seed
              integer for an identical request will produce similar results.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/character_performance",
            body=await async_maybe_transform(
                {
                    "character": character,
                    "model": model,
                    "reference": reference,
                    "body_control": body_control,
                    "content_moderation": content_moderation,
                    "expression_intensity": expression_intensity,
                    "ratio": ratio,
                    "seed": seed,
                },
                character_performance_create_params.CharacterPerformanceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=create_async_waitable_resource(CharacterPerformanceCreateResponse, self._client),
        )


class CharacterPerformanceResourceWithRawResponse:
    def __init__(self, character_performance: CharacterPerformanceResource) -> None:
        self._character_performance = character_performance

        self.create = to_raw_response_wrapper(
            character_performance.create,
        )


class AsyncCharacterPerformanceResourceWithRawResponse:
    def __init__(self, character_performance: AsyncCharacterPerformanceResource) -> None:
        self._character_performance = character_performance

        self.create = async_to_raw_response_wrapper(
            character_performance.create,
        )


class CharacterPerformanceResourceWithStreamingResponse:
    def __init__(self, character_performance: CharacterPerformanceResource) -> None:
        self._character_performance = character_performance

        self.create = to_streamed_response_wrapper(
            character_performance.create,
        )


class AsyncCharacterPerformanceResourceWithStreamingResponse:
    def __init__(self, character_performance: AsyncCharacterPerformanceResource) -> None:
        self._character_performance = character_performance

        self.create = async_to_streamed_response_wrapper(
            character_performance.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = ["TextToVideoCreateParams", "Gen4_5", "Gen4_5ContentModeration", "Veo3_1", "Veo3_1Fast", "Veo3"]


class Gen4_5(TypedDict, total=False):
    duration: Required[int]
    """The number of seconds of duration for the output video.

    Must be an integer from 2 to 10.
    """

    model: Required[Literal["gen4.5"]]

    prompt_text: Required[Annotated[str, PropertyInfo(alias="promptText")]]
    """A non-empty string up to 1000 characters (measured in UTF-16 code units).

    This should describe in detail what should appear in the output.
    """

    ratio: Required[Literal["1280:720", "720:1280"]]
    """The resolution of the output video."""

    content_moderation: Annotated[Gen4_5ContentModeration, PropertyInfo(alias="contentModeration")]
    """Settings that affect the behavior of the content moderation system."""

    seed: int
    """If unspecified, a random number is chosen.

    Varying the seed integer is a way to get different results for the same other
    request parameters. Using the same seed integer for an identical request will
    produce similar results.
    """


class Gen4_5ContentModeration(TypedDict, total=False):
    """Settings that affect the behavior of the content moderation system."""

    public_figure_threshold: Annotated[Literal["auto", "low"], PropertyInfo(alias="publicFigureThreshold")]
    """
    When set to `low`, the content moderation system will be less strict about
    preventing generations that include recognizable public figures.
    """


class Veo3_1(TypedDict, total=False):
    model: Required[Literal["veo3.1"]]

    prompt_text: Required[Annotated[str, PropertyInfo(alias="promptText")]]
    """A non-empty string up to 1000 characters (measured in UTF-16 code units).

    This should describe in detail what should appear in the output.
    """

    ratio: Required[Literal["1280:720", "720:1280", "1080:1920", "1920:1080"]]
    """The resolution of the output video."""

    audio: bool
    """Whether to generate audio for the video. Audio inclusion affects pricing."""

    duration: Literal[4, 6, 8]
    """The number of seconds of duration for the output video."""


class Veo3_1Fast(TypedDict, total=False):
    model: Required[Literal["veo3.1_fast"]]

    prompt_text: Required[Annotated[str, PropertyInfo(alias="promptText")]]
    """A non-empty string up to 1000 characters (measured in UTF-16 code units).

    This should describe in detail what should appear in the output.
    """

    ratio: Required[Literal["1280:720", "720:1280", "1080:1920", "1920:1080"]]
    """The resolution of the output video."""

    audio: bool
    """Whether to generate audio for the video. Audio inclusion affects pricing."""

    duration: Literal[4, 6, 8]
    """The number of seconds of duration for the output video."""


class Veo3(TypedDict, total=False):
    duration: Required[Literal[8]]
    """The number of seconds of duration for the output video."""

    model: Required[Literal["veo3"]]

    prompt_text: Required[Annotated[str, PropertyInfo(alias="promptText")]]
    """A non-empty string up to 1000 characters (measured in UTF-16 code units).

    This should describe in detail what should appear in the output.
    """

    ratio: Required[Literal["1280:720", "720:1280", "1080:1920", "1920:1080"]]
    """The resolution of the output video."""


TextToVideoCreateParams: TypeAlias = Union[Gen4_5, Veo3_1, Veo3_1Fast, Veo3]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from pydantic import Field as FieldInfo

from .._utils import PropertyInfo
from .._models import BaseModel

__all__ = ["TaskRetrieveResponse", "Pending", "Throttled", "Cancelled", "Running", "Failed", "Succeeded"]


class Pending(BaseModel):
    """A pending task"""

    id: str
    """The ID of the task being returned."""

    created_at: datetime = FieldInfo(alias="createdAt")
    """The timestamp that the task was submitted at."""

    status: Literal["PENDING"]


class Throttled(BaseModel):
    """A throttled task"""

    id: str
    """The ID of the task being returned."""

    created_at: datetime = FieldInfo(alias="createdAt")
    """The timestamp that the task was submitted at."""

    status: Literal["THROTTLED"]


class Cancelled(BaseModel):
    """A cancelled or deleted task"""

    id: str
    """The ID of the task being returned."""

    created_at: datetime = FieldInfo(alias="createdAt")
    """The timestamp that the task was submitted at."""

    status: Literal["CANCELLED"]


class Running(BaseModel):
    """A running task"""

    id: str
    """The ID of the task being returned."""

    created_at: datetime = FieldInfo(alias="createdAt")
    """The timestamp that the task was submitted at."""

    progress: float

    status: Literal["RUNNING"]


class Failed(BaseModel):
    """A failed task"""

    id: str
    """The ID of the task being returned."""

    created_at: datetime = FieldInfo(alias="createdAt")
    """The timestamp that the task was submitted at."""

    failure: str
    """A human-friendly reason for the failure.

    We do not recommend returning this to users directly without adding context.
    """

    status: Literal["FAILED"]

    failure_code: Optional[str] = FieldInfo(alias="failureCode", default=None)
    """A machine-readable error code for the failure.

    See https://docs.dev.runwayml.com/errors/task-failures/ for more information.
    """


class Succeeded(BaseModel):
    """A succeeded task"""

    id: str
    """The ID of the task being returned."""

    created_at: datetime = FieldInfo(alias="createdAt")
    """The timestamp that the task was submitted at."""

    output: List[str]
    """An array of URLs that return the output of the task.

    These URLs will expire within 24-48 hours; fetch the task again to get fresh
    URLs. It is expected that you download the assets at these URLs and store them
    in your own storage system.
    """

    status: Literal["SUCCEEDED"]


TaskRetrieveResponse: TypeAlias = Annotated[
    Union[Pending, Throttled, Cancelled, Running, Failed, Succeeded], PropertyInfo(discriminator="status")
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Literal, Mapping

import httpx
from httpx import Timeout

from runwayml._models import BaseModel

from .._files import _transform_file, _async_transform_file
from .._types import Body, Omit, Query, Headers, NotGiven, FileTypes, HttpxFileTypes, omit, not_given
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._exceptions import APIStatusError, APITimeoutError, APIConnectionError
from .._base_client import make_request_options
from ..types.upload_create_ephemeral_response import UploadCreateEphemeralResponse


class _RunwayUploadStartedResponse(BaseModel):
    runwayUri: str
    uploadUrl: str
    fields: Mapping[str, str]


class _RunwayCreateUploadArguments(BaseModel):
    filename: str
    type: Literal["ephemeral"] = "ephemeral"


def _infer_filename(file: FileTypes) -> str:
    """Infer filename from FileTypes. Raises ValueError if filename cannot be inferred."""
    # If it's a tuple, use the first element as filename
    if isinstance(file, tuple):
        filename = file[0]
        if filename is not None:
            return filename
        # If filename is None in tuple, try to infer from the file content
        file_content = file[1]
    else:
        file_content = file

    # If it's a PathLike, extract the basename
    if isinstance(file_content, os.PathLike):
        return os.path.basename(str(file_content))

    # If it's bytes, we cannot infer
    if isinstance(file_content, bytes):
        raise ValueError(
            "Cannot infer filename from file. Please use the tuple format: "
            "(filename, file_content) where filename is a string. "
            "For example: ('myfile.jpg', file_content)"
        )

    # If it's an IO object, try to get the name attribute
    name = getattr(file_content, "name", None)
    if name is not None and isinstance(name, str):
        return str(os.path.basename(name))

    # If it's an IO object without a name, we cannot infer
    raise ValueError(
        "Cannot infer filename from file. Please use the tuple format: "
        "(filename, file_content) where filename is a string. "
        "For example: ('myfile.jpg', file_content)"
    )


def _build_file_tuple_with_filename(filename: str, transformed_file: HttpxFileTypes) -> HttpxFileTypes:
    """Build a file tuple with the given filename, preserving any additional metadata."""
    if isinstance(transformed_file, tuple):
        # Extract content and any additional elements (content_type, headers)
        file_content = transformed_file[1]
        if len(transformed_file) == 2:
            # (filename, content)
            return (filename, file_content)
        elif len(transformed_file) == 3:
            # (filename, content, content_type)
            return (filename, file_content, transformed_file[2])
        else:  # len == 4
            # (filename, content, content_type, headers)
            return (filename, file_content, transformed_file[2], transformed_file[3])
    else:
        # Just content, wrap with filename
        return (filename, transformed_file)


def _prepare_upload_data(
    fields: Mapping[str, str],
    file_metadata: str | Omit,
) -> dict[str, str]:
    """Prepare form data for upload, including metadata if provided."""
    form_data = dict(fields)
    if not isinstance(file_metadata, Omit):
        form_data["metadata"] = file_metadata
    return form_data


def _get_upload_timeout(
    timeout: float | httpx.Timeout | None | NotGiven,
    client_timeout: float | Timeout | None,
) -> float | Timeout | None:
    """Extract timeout for upload request."""
    if isinstance(timeout, NotGiven):
        return client_timeout
    return timeout


def _handle_upload_errors(response: httpx.Response, upload_url: str) -> None:
    """Handle upload errors consistently."""
    upload_request = httpx.Request("POST", httpx.URL(upload_url))
    try:
        response.raise_for_status()
    except httpx.TimeoutException as err:
        request = getattr(err, "request", upload_request)
        raise APITimeoutError(request=request) from err
    except httpx.RequestError as err:
        request = getattr(err, "request", upload_request)
        raise APIConnectionError(message=str(err), request=request) from err
    except httpx.HTTPStatusError as err:
        raise APIStatusError(
            message=f"Upload failed with status {err.response.status_code}",
            response=err.response,
            body=err.response.text,
        ) from err


__all__ = ["UploadsResource", "AsyncUploadsResource"]


class UploadsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UploadsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return UploadsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UploadsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return UploadsResourceWithStreamingResponse(self)

    def create_ephemeral(
        self,
        *,
        file: FileTypes,
        file_metadata: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UploadCreateEphemeralResponse:
        """Uploads a temporary media file that can be used in other API generation
        requests.

        The uploaded files will be automatically expired and deleted after a
        period of time.

        Args:
          file: The file to upload. Must be a supported media type (image, video, or audio).

          file_metadata: Optional metadata for the file, serialized as a field in multipart/form-data.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        filename = _infer_filename(file)
        upload_placeholder = self._post(
            "/v1/uploads",
            body=maybe_transform(
                _RunwayCreateUploadArguments(filename=filename, type="ephemeral"),
                _RunwayCreateUploadArguments,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=_RunwayUploadStartedResponse,
        )

        file_tuple = _build_file_tuple_with_filename(filename, _transform_file(file))
        form_data = _prepare_upload_data(upload_placeholder.fields, file_metadata)
        upload_timeout = _get_upload_timeout(timeout, self._client.timeout)

        with httpx.Client(timeout=upload_timeout) as client:
            response = client.post(
                upload_placeholder.uploadUrl,
                data=form_data,
                files={"file": file_tuple},
            )
            _handle_upload_errors(response, upload_placeholder.uploadUrl)

        return UploadCreateEphemeralResponse(uri=upload_placeholder.runwayUri)


class AsyncUploadsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUploadsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUploadsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUploadsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return AsyncUploadsResourceWithStreamingResponse(self)

    async def create_ephemeral(
        self,
        *,
        file: FileTypes,
        file_metadata: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UploadCreateEphemeralResponse:
        """Uploads a temporary media file that can be used in other API generation
        requests.

        The uploaded files will be automatically expired and deleted after a
        period of time.

        Args:
          file: The file to upload. Must be a supported media type (image, video, or audio).

          file_metadata: Optional metadata for the file, serialized as a field in multipart/form-data.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        filename = _infer_filename(file)
        upload_placeholder = await self._post(
            "/v1/uploads",
            body=maybe_transform(
                _RunwayCreateUploadArguments(filename=filename, type="ephemeral"),
                _RunwayCreateUploadArguments,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=_RunwayUploadStartedResponse,
        )

        file_tuple = _build_file_tuple_with_filename(filename, await _async_transform_file(file))
        form_data = _prepare_upload_data(upload_placeholder.fields, file_metadata)
        upload_timeout = _get_upload_timeout(timeout, self._client.timeout)

        async with httpx.AsyncClient(timeout=upload_timeout) as client:
            response = await client.post(
                upload_placeholder.uploadUrl,
                data=form_data,
                files={"file": file_tuple},
            )
            _handle_upload_errors(response, upload_placeholder.uploadUrl)

        return UploadCreateEphemeralResponse(uri=upload_placeholder.runwayUri)


class UploadsResourceWithRawResponse:
    def __init__(self, uploads: UploadsResource) -> None:
        self._uploads = uploads

        self.create_ephemeral = to_raw_response_wrapper(
            uploads.create_ephemeral,
        )


class AsyncUploadsResourceWithRawResponse:
    def __init__(self, uploads: AsyncUploadsResource) -> None:
        self._uploads = uploads

        self.create_ephemeral = async_to_raw_response_wrapper(
            uploads.create_ephemeral,
        )


class UploadsResourceWithStreamingResponse:
    def __init__(self, uploads: UploadsResource) -> None:
        self._uploads = uploads

        self.create_ephemeral = to_streamed_response_wrapper(
            uploads.create_ephemeral,
        )


class AsyncUploadsResourceWithStreamingResponse:
    def __init__(self, uploads: AsyncUploadsResource) -> None:
        self._uploads = uploads

        self.create_ephemeral = async_to_streamed_response_wrapper(
            uploads.create_ephemeral,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["VideoToVideoCreateParams", "ContentModeration", "Reference"]


class VideoToVideoCreateParams(TypedDict, total=False):
    model: Required[Literal["gen4_aleph"]]

    prompt_text: Required[Annotated[str, PropertyInfo(alias="promptText")]]
    """A non-empty string up to 1000 characters (measured in UTF-16 code units).

    This should describe in detail what should appear in the output.
    """

    video_uri: Required[Annotated[str, PropertyInfo(alias="videoUri")]]
    """A HTTPS URL."""

    content_moderation: Annotated[ContentModeration, PropertyInfo(alias="contentModeration")]
    """Settings that affect the behavior of the content moderation system."""

    ratio: Literal["1280:720", "720:1280", "1104:832", "960:960", "832:1104", "1584:672", "848:480", "640:480"]
    """Deprecated.

    This field is ignored. The resolution of the output video is determined by the
    input video.
    """

    references: Iterable[Reference]
    """An array of references.

    Currently up to one reference is supported. See
    [our docs](/assets/inputs#images) on image inputs for more information.
    """

    seed: int
    """If unspecified, a random number is chosen.

    Varying the seed integer is a way to get different results for the same other
    request parameters. Using the same seed integer for an identical request will
    produce similar results.
    """


class ContentModeration(TypedDict, total=False):
    """Settings that affect the behavior of the content moderation system."""

    public_figure_threshold: Annotated[Literal["auto", "low"], PropertyInfo(alias="publicFigureThreshold")]
    """
    When set to `low`, the content moderation system will be less strict about
    preventing generations that include recognizable public figures.
    """


class Reference(TypedDict, total=False):
    """
    Passing an image reference allows the model to emulate the style or content of the reference in the output.
    """

    type: Required[Literal["image"]]

    uri: Required[str]
    """A HTTPS URL."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = ["SpeechToSpeechCreateParams", "Media", "MediaAudio", "MediaVideo", "Voice"]


class SpeechToSpeechCreateParams(TypedDict, total=False):
    media: Required[Media]
    """An audio file containing dialogue to be processed."""

    model: Required[Literal["eleven_multilingual_sts_v2"]]

    voice: Required[Voice]
    """A voice preset from the RunwayML API."""

    remove_background_noise: Annotated[bool, PropertyInfo(alias="removeBackgroundNoise")]
    """Whether to remove background noise from the generated speech."""


class MediaAudio(TypedDict, total=False):
    """An audio file containing dialogue to be processed."""

    type: Required[Literal["audio"]]

    uri: Required[str]
    """A HTTPS URL."""


class MediaVideo(TypedDict, total=False):
    """A video file containing dialogue to be processed."""

    type: Required[Literal["video"]]

    uri: Required[str]
    """A HTTPS URL."""


Media: TypeAlias = Union[MediaAudio, MediaVideo]


class Voice(TypedDict, total=False):
    """A voice preset from the RunwayML API."""

    preset_id: Required[
        Annotated[
            Literal[
                "Maya",
                "Arjun",
                "Serene",
                "Bernard",
                "Billy",
                "Mark",
                "Clint",
                "Mabel",
                "Chad",
                "Leslie",
                "Eleanor",
                "Elias",
                "Elliot",
                "Grungle",
                "Brodie",
                "Sandra",
                "Kirk",
                "Kylie",
                "Lara",
                "Lisa",
                "Malachi",
                "Marlene",
                "Martin",
                "Miriam",
                "Monster",
                "Paula",
                "Pip",
                "Rusty",
                "Ragnar",
                "Xylar",
                "Maggie",
                "Jack",
                "Katie",
                "Noah",
                "James",
                "Rina",
                "Ella",
                "Mariah",
                "Frank",
                "Claudia",
                "Niki",
                "Vincent",
                "Kendrick",
                "Myrna",
                "Tom",
                "Wanda",
                "Benjamin",
                "Kiana",
                "Rachel",
            ],
            PropertyInfo(alias="presetId"),
        ]
    ]
    """The preset voice ID to use for the generated speech."""

    type: Required[Literal["runway-preset"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["VoiceIsolationCreateParams"]


class VoiceIsolationCreateParams(TypedDict, total=False):
    audio_uri: Required[Annotated[str, PropertyInfo(alias="audioUri")]]
    """A HTTPS URL."""

    model: Required[Literal["eleven_voice_isolation"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import datetime
from typing import List
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["OrganizationRetrieveUsageResponse", "Result", "ResultUsedCredit"]


class ResultUsedCredit(BaseModel):
    amount: int
    """The number of credits spent on the model."""

    model: Literal[
        "gen4.5",
        "gen3a_turbo",
        "gen4_turbo",
        "gen4_image",
        "gen4_image_turbo",
        "act_two",
        "gen4_aleph",
        "veo3",
        "veo3.1",
        "veo3.1_fast",
        "gemini_2.5_flash",
        "eleven_multilingual_v2",
        "eleven_text_to_sound_v2",
        "eleven_voice_isolation",
        "eleven_voice_dubbing",
        "eleven_multilingual_sts_v2",
    ]
    """The model that credits were spent on."""


class Result(BaseModel):
    date: datetime.date
    """The date of the usage data in ISO-8601 format (YYYY-MM-DD).

    All dates are in UTC.
    """

    used_credits: List[ResultUsedCredit] = FieldInfo(alias="usedCredits")
    """The credits used per model for the given date."""


class OrganizationRetrieveUsageResponse(BaseModel):
    models: List[
        Literal[
            "gen4.5",
            "gen3a_turbo",
            "gen4_turbo",
            "gen4_image",
            "gen4_image_turbo",
            "act_two",
            "gen4_aleph",
            "veo3",
            "veo3.1",
            "veo3.1_fast",
            "gemini_2.5_flash",
            "eleven_multilingual_v2",
            "eleven_text_to_sound_v2",
            "eleven_voice_isolation",
            "eleven_voice_dubbing",
            "eleven_multilingual_sts_v2",
        ]
    ]
    """The list of models with usage during the queried time range."""

    results: List[Result]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "CharacterPerformanceCreateParams",
    "Character",
    "CharacterImage",
    "CharacterVideo",
    "Reference",
    "ContentModeration",
]


class CharacterPerformanceCreateParams(TypedDict, total=False):
    character: Required[Character]
    """The character to control.

    You can either provide a video or an image. A visually recognizable face must be
    visible and stay within the frame.
    """

    model: Required[Literal["act_two"]]

    reference: Required[Reference]
    """
    A video of a person performing in the manner that you would like your character
    to perform. The video must be between 3 and 30 seconds in duration.
    """

    body_control: Annotated[bool, PropertyInfo(alias="bodyControl")]
    """A boolean indicating whether to enable body control.

    When enabled, non-facial movements and gestures will be applied to the character
    in addition to facial expressions.
    """

    content_moderation: Annotated[ContentModeration, PropertyInfo(alias="contentModeration")]
    """Settings that affect the behavior of the content moderation system."""

    expression_intensity: Annotated[int, PropertyInfo(alias="expressionIntensity")]
    """An integer between 1 and 5 (inclusive).

    A larger value increases the intensity of the character's expression.
    """

    ratio: Literal["1280:720", "720:1280", "960:960", "1104:832", "832:1104", "1584:672"]
    """The resolution of the output video."""

    seed: int
    """If unspecified, a random number is chosen.

    Varying the seed integer is a way to get different results for the same other
    request parameters. Using the same seed integer for an identical request will
    produce similar results.
    """


class CharacterImage(TypedDict, total=False):
    """An image of your character.

    In the output, the character will use the reference video performance in its original static environment.
    """

    type: Required[Literal["image"]]

    uri: Required[str]
    """A HTTPS URL."""


class CharacterVideo(TypedDict, total=False):
    """A video of your character.

    In the output, the character will use the reference video performance in its original animated environment and some of the character's own movements.
    """

    type: Required[Literal["video"]]

    uri: Required[str]
    """A HTTPS URL."""


Character: TypeAlias = Union[CharacterImage, CharacterVideo]


class Reference(TypedDict, total=False):
    """
    A video of a person performing in the manner that you would like your character to perform. The video must be between 3 and 30 seconds in duration.
    """

    type: Required[Literal["video"]]

    uri: Required[str]
    """A HTTPS URL."""


class ContentModeration(TypedDict, total=False):
    """Settings that affect the behavior of the content moderation system."""

    public_figure_threshold: Annotated[Literal["auto", "low"], PropertyInfo(alias="publicFigureThreshold")]
    """
    When set to `low`, the content moderation system will be less strict about
    preventing generations that include recognizable public figures.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date

import httpx

from ..types import organization_retrieve_usage_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.organization_retrieve_response import OrganizationRetrieveResponse
from ..types.organization_retrieve_usage_response import OrganizationRetrieveUsageResponse

__all__ = ["OrganizationResource", "AsyncOrganizationResource"]


class OrganizationResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OrganizationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return OrganizationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OrganizationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return OrganizationResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OrganizationRetrieveResponse:
        """
        Get usage tier and credit balance information about the organization associated
        with the API key used to make the request.
        """
        return self._get(
            "/v1/organization",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationRetrieveResponse,
        )

    def retrieve_usage(
        self,
        *,
        before_date: Union[str, date] | Omit = omit,
        start_date: Union[str, date] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OrganizationRetrieveUsageResponse:
        """
        Fetch credit usage data broken down by model and day for the organization
        associated with the API key used to make the request. Up to 90 days of data can
        be queried at a time.

        Args:
          before_date: The end date of the usage data in ISO-8601 format (YYYY-MM-DD), not inclusive.
              If unspecified, it will default to thirty days after the start date. Must be
              less than or equal to 90 days after the start date. All dates are in UTC.

          start_date: The start date of the usage data in ISO-8601 format (YYYY-MM-DD). If
              unspecified, it will default to 30 days before the current date. All dates are
              in UTC.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/organization/usage",
            body=maybe_transform(
                {
                    "before_date": before_date,
                    "start_date": start_date,
                },
                organization_retrieve_usage_params.OrganizationRetrieveUsageParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationRetrieveUsageResponse,
        )


class AsyncOrganizationResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOrganizationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOrganizationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOrganizationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return AsyncOrganizationResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OrganizationRetrieveResponse:
        """
        Get usage tier and credit balance information about the organization associated
        with the API key used to make the request.
        """
        return await self._get(
            "/v1/organization",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationRetrieveResponse,
        )

    async def retrieve_usage(
        self,
        *,
        before_date: Union[str, date] | Omit = omit,
        start_date: Union[str, date] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OrganizationRetrieveUsageResponse:
        """
        Fetch credit usage data broken down by model and day for the organization
        associated with the API key used to make the request. Up to 90 days of data can
        be queried at a time.

        Args:
          before_date: The end date of the usage data in ISO-8601 format (YYYY-MM-DD), not inclusive.
              If unspecified, it will default to thirty days after the start date. Must be
              less than or equal to 90 days after the start date. All dates are in UTC.

          start_date: The start date of the usage data in ISO-8601 format (YYYY-MM-DD). If
              unspecified, it will default to 30 days before the current date. All dates are
              in UTC.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/organization/usage",
            body=await async_maybe_transform(
                {
                    "before_date": before_date,
                    "start_date": start_date,
                },
                organization_retrieve_usage_params.OrganizationRetrieveUsageParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=OrganizationRetrieveUsageResponse,
        )


class OrganizationResourceWithRawResponse:
    def __init__(self, organization: OrganizationResource) -> None:
        self._organization = organization

        self.retrieve = to_raw_response_wrapper(
            organization.retrieve,
        )
        self.retrieve_usage = to_raw_response_wrapper(
            organization.retrieve_usage,
        )


class AsyncOrganizationResourceWithRawResponse:
    def __init__(self, organization: AsyncOrganizationResource) -> None:
        self._organization = organization

        self.retrieve = async_to_raw_response_wrapper(
            organization.retrieve,
        )
        self.retrieve_usage = async_to_raw_response_wrapper(
            organization.retrieve_usage,
        )


class OrganizationResourceWithStreamingResponse:
    def __init__(self, organization: OrganizationResource) -> None:
        self._organization = organization

        self.retrieve = to_streamed_response_wrapper(
            organization.retrieve,
        )
        self.retrieve_usage = to_streamed_response_wrapper(
            organization.retrieve_usage,
        )


class AsyncOrganizationResourceWithStreamingResponse:
    def __init__(self, organization: AsyncOrganizationResource) -> None:
        self._organization = organization

        self.retrieve = async_to_streamed_response_wrapper(
            organization.retrieve,
        )
        self.retrieve_usage = async_to_streamed_response_wrapper(
            organization.retrieve_usage,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["SoundEffectCreateParams"]


class SoundEffectCreateParams(TypedDict, total=False):
    model: Required[Literal["eleven_text_to_sound_v2"]]

    prompt_text: Required[Annotated[str, PropertyInfo(alias="promptText")]]
    """A text description of the sound effect to generate."""

    duration: float
    """The duration of the sound effect in seconds, between 0.5 and 30 seconds.

    If not provided, the duration will be determined automatically based on the text
    description.
    """

    loop: bool
    """Whether the output sound effect should be designed to loop seamlessly."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, cast

import httpx

from .._types import Body, Query, Headers, NoneType, NotGiven, not_given
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..lib.polling import (
    AwaitableTaskRetrieveResponse,
    AsyncAwaitableTaskRetrieveResponse,
    inject_sync_wait_method,
    inject_async_wait_method,
)
from .._base_client import make_request_options

__all__ = ["TasksResource", "AsyncTasksResource"]


class TasksResource(SyncAPIResource):
    """Endpoints for managing tasks that have been submitted."""

    @cached_property
    def with_raw_response(self) -> TasksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return TasksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TasksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return TasksResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AwaitableTaskRetrieveResponse:
        """Return details about a task.

        Consumers of this API should not expect updates
        more frequent than once every five seconds for a given task.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        response = self._get(
            f"/v1/tasks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=cast(
                Any, AwaitableTaskRetrieveResponse
            ),  # Union types cannot be passed in as arguments in the type system
        )
        return cast(AwaitableTaskRetrieveResponse, inject_sync_wait_method(self._client, response))

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Tasks that are running, pending, or throttled can be canceled by invoking this
        method. Invoking this method for other tasks will delete them.

        The output data associated with a deleted task will be deleted from persistent
        storage in accordance with our data retention policy. Aborted and deleted tasks
        will not be able to be fetched again in the future.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v1/tasks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncTasksResource(AsyncAPIResource):
    """Endpoints for managing tasks that have been submitted."""

    @cached_property
    def with_raw_response(self) -> AsyncTasksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTasksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTasksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return AsyncTasksResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncAwaitableTaskRetrieveResponse:
        """Return details about a task.

        Consumers of this API should not expect updates
        more frequent than once every five seconds for a given task.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        response = await self._get(
            f"/v1/tasks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=cast(
                Any, AsyncAwaitableTaskRetrieveResponse
            ),  # Union types cannot be passed in as arguments in the type system
        )
        return cast(AsyncAwaitableTaskRetrieveResponse, inject_async_wait_method(self._client, response))

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Tasks that are running, pending, or throttled can be canceled by invoking this
        method. Invoking this method for other tasks will delete them.

        The output data associated with a deleted task will be deleted from persistent
        storage in accordance with our data retention policy. Aborted and deleted tasks
        will not be able to be fetched again in the future.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v1/tasks/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class TasksResourceWithRawResponse:
    def __init__(self, tasks: TasksResource) -> None:
        self._tasks = tasks

        self.retrieve = to_raw_response_wrapper(
            tasks.retrieve,
        )
        self.delete = to_raw_response_wrapper(
            tasks.delete,
        )


class AsyncTasksResourceWithRawResponse:
    def __init__(self, tasks: AsyncTasksResource) -> None:
        self._tasks = tasks

        self.retrieve = async_to_raw_response_wrapper(
            tasks.retrieve,
        )
        self.delete = async_to_raw_response_wrapper(
            tasks.delete,
        )


class TasksResourceWithStreamingResponse:
    def __init__(self, tasks: TasksResource) -> None:
        self._tasks = tasks

        self.retrieve = to_streamed_response_wrapper(
            tasks.retrieve,
        )
        self.delete = to_streamed_response_wrapper(
            tasks.delete,
        )


class AsyncTasksResourceWithStreamingResponse:
    def __init__(self, tasks: AsyncTasksResource) -> None:
        self._tasks = tasks

        self.retrieve = async_to_streamed_response_wrapper(
            tasks.retrieve,
        )
        self.delete = async_to_streamed_response_wrapper(
            tasks.delete,
        )



from runwayml import RunwayML, TaskFailedError

client = RunwayML()

image_task = client.text_to_image.create(
    model="gen4_image",
    prompt_text="A beautiful sunset over a calm ocean",
    ratio="1920:1080",
)
try:
    output = image_task.wait_for_task_output()
except TaskFailedError as e:
    print("The image failed to generate.")
    print(e.task_details)
else:
    print(output.output[0])  # type: ignore



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import speech_to_speech_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..lib.polling import (
    NewTaskCreatedResponse,
    AsyncNewTaskCreatedResponse,
    create_waitable_resource,
    create_async_waitable_resource,
)
from .._base_client import make_request_options
from ..types.speech_to_speech_create_response import SpeechToSpeechCreateResponse

__all__ = ["SpeechToSpeechResource", "AsyncSpeechToSpeechResource"]


class SpeechToSpeechResource(SyncAPIResource):
    """These endpoints all kick off tasks to create generations."""

    @cached_property
    def with_raw_response(self) -> SpeechToSpeechResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return SpeechToSpeechResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SpeechToSpeechResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return SpeechToSpeechResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        media: speech_to_speech_create_params.Media,
        model: Literal["eleven_multilingual_sts_v2"],
        voice: speech_to_speech_create_params.Voice,
        remove_background_noise: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NewTaskCreatedResponse:
        """
        This endpoint will start a new task to convert speech from one voice to another
        in audio or video.

        Args:
          media: An audio file containing dialogue to be processed.

          voice: A voice preset from the RunwayML API.

          remove_background_noise: Whether to remove background noise from the generated speech.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/speech_to_speech",
            body=maybe_transform(
                {
                    "media": media,
                    "model": model,
                    "voice": voice,
                    "remove_background_noise": remove_background_noise,
                },
                speech_to_speech_create_params.SpeechToSpeechCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=create_waitable_resource(SpeechToSpeechCreateResponse, self._client),
        )


class AsyncSpeechToSpeechResource(AsyncAPIResource):
    """These endpoints all kick off tasks to create generations."""

    @cached_property
    def with_raw_response(self) -> AsyncSpeechToSpeechResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/runwayml/sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSpeechToSpeechResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(
        self,
    ) -> AsyncSpeechToSpeechResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/runwayml/sdk-python#with_streaming_response
        """
        return AsyncSpeechToSpeechResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        media: speech_to_speech_create_params.Media,
        model: Literal["eleven_multilingual_sts_v2"],
        voice: speech_to_speech_create_params.Voice,
        remove_background_noise: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncNewTaskCreatedResponse:
        """
        This endpoint will start a new task to convert speech from one voice to another
        in audio or video.

        Args:
          media: An audio file containing dialogue to be processed.

          voice: A voice preset from the RunwayML API.

          remove_background_noise: Whether to remove background noise from the generated speech.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/speech_to_speech",
            body=await async_maybe_transform(
                {
                    "media": media,
                    "model": model,
                    "voice": voice,
                    "remove_background_noise": remove_background_noise,
                },
                speech_to_speech_create_params.SpeechToSpeechCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            ),
            cast_to=create_async_waitable_resource(SpeechToSpeechCreateResponse, self._client),
        )


class SpeechToSpeechResourceWithRawResponse:
    def __init__(self, speech_to_speech: SpeechToSpeechResource) -> None:
        self._speech_to_speech = speech_to_speech

        self.create = to_raw_response_wrapper(
            speech_to_speech.create,
        )


class AsyncSpeechToSpeechResourceWithRawResponse:
    def __init__(self, speech_to_speech: AsyncSpeechToSpeechResource) -> None:
        self._speech_to_speech = speech_to_speech

        self.create = async_to_raw_response_wrapper(
            speech_to_speech.create,
        )


class SpeechToSpeechResourceWithStreamingResponse:
    def __init__(self, speech_to_speech: SpeechToSpeechResource) -> None:
        self._speech_to_speech = speech_to_speech

        self.create = to_streamed_response_wrapper(
            speech_to_speech.create,
        )


class AsyncSpeechToSpeechResourceWithStreamingResponse:
    def __init__(self, speech_to_speech: AsyncSpeechToSpeechResource) -> None:
        self._speech_to_speech = speech_to_speech

        self.create = async_to_streamed_response_wrapper(
            speech_to_speech.create,
        )
